/* Questions.java
 * ------------------------------------------------------------------ */
package net.esclat.ripple;

public class Utility {
    private static char[] hex = {'0','1','2','3','4','5','6','7',
                                 '8','9','A','B','C','D','E','F'};
    /** Converts a byte array to a hexidecimal string. */
    public static String hexify(byte[] data) {
        StringBuilder out = new StringBuilder();
        for (byte b : data)
            out.append(hex[(b >> 4) & 0xF]).append(hex[b & 0xF]);
        return out.toString();
    }
    /** Converts a hexidecimal string to a byte array. */
    public static byte[] dehexify(String data) {
        byte[] dest = new byte[(data.length() + 1) / 2];
        int    didx = 0;
        for (char c : data.toCharArray()) {
            c = Character.toLowerCase(c);
            if ((c >= '0') && (c <= '9')) {
                if ((didx % 2) == 0)
                    dest[didx++/2] = (byte)((c - '0') << 4);
                else dest[didx++/2] |= c - '0';
            } else if ((c >= 'A') && (c <= 'F')) {
                if ((didx % 2) == 0)
                    dest[didx++/2] = (byte)((c - 'A' + 10) << 4);
                else dest[didx++/2] |= c - 'A' + 10;
            } else return null;
        }
        return dest;
    }

    /** Provides answers about the Java environment.  Each argument is
     *  a request and is preceeded with a prefix that determines what
     *  the question is.  A <q>p</q> request fetches the value of the
     *  system property specified after a colon.  A <q>c</q> request
     *  checks whether the class named after the colon can be loaded.
     *  For example:
     *
     *  <pre>
     *  $ java com.ncipher.ps.tools.Questions p:java.home       \
     *      c:java.lang.String c:java.lang.Missing z:nonsense
     *  p:java.home =&gt; "/usr/lib/jvm/java-6-openjdk/jre"
     *  c:java.lang.String =&gt; java.lang.String
     *  c:java.lang.Missing =&gt; &lt;missing&gt;
     *  z:nonsense =&gt; &lt;unknown request&gt;
     *  </pre> */
    public static void main(String[] args) {
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            System.out.print(arg);
            System.out.print(" => ");                
            if (arg.startsWith("p:")) {
                String value = System.getProperty(arg.substring(2));
                System.out.println((value != null) ?
                                   "\"" + value + "\"" : "null");
            } else if (arg.startsWith("c:")) {
                Class found = null;
                try {
                    found = Class.forName(arg.substring(2));
                } catch (ClassNotFoundException cnfe) {}
                System.out.println((found != null) ?
                                   found.getName() : "<missing>");
            } else System.out.println("<unknown request>");
        }
    }
}
